
# generated by tools/generate.py
# https://github.com/esoma/pyglm-typing/

from typing import SupportsFloat, SupportsInt, Tuple, Union

from . import glm

Number = Union[SupportsFloat, SupportsInt]
B8Vector1 = Union[glm.bvec1, Tuple[Number]]
B8Vec1 = glm.bvec1
B8Vector2 = Union[glm.bvec2, Tuple[Number, Number]]
B8Vec2 = glm.bvec2
B8Vector3 = Union[glm.bvec3, Tuple[Number, Number, Number]]
B8Vec3 = glm.bvec3
B8Vector4 = Union[glm.bvec4, Tuple[Number, Number, Number, Number]]
B8Vec4 = glm.bvec4
B8VectorAny = Union[glm.bvec1, glm.bvec2, glm.bvec3, glm.bvec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
B8VecAny = Union[glm.bvec1, glm.bvec2, glm.bvec3, glm.bvec4]
BAnyVector1 = Union[glm.bvec1, Tuple[Number]]
BAnyVec1 = glm.bvec1
BAnyVector2 = Union[glm.bvec2, Tuple[Number, Number]]
BAnyVec2 = glm.bvec2
BAnyVector3 = Union[glm.bvec3, Tuple[Number, Number, Number]]
BAnyVec3 = glm.bvec3
BAnyVector4 = Union[glm.bvec4, Tuple[Number, Number, Number, Number]]
BAnyVec4 = glm.bvec4
BAnyVectorAny = Union[glm.bvec1, glm.bvec2, glm.bvec3, glm.bvec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
BAnyVecAny = Union[glm.bvec1, glm.bvec2, glm.bvec3, glm.bvec4]
D64Vector1 = Union[glm.dvec1, Tuple[Number]]
D64Vec1 = glm.dvec1
D64Vector2 = Union[glm.dmvec2, glm.dvec2, Tuple[Number, Number]]
D64Vec2 = Union[glm.dmvec2, glm.dvec2]
D64Vector3 = Union[glm.dmvec3, glm.dvec3, Tuple[Number, Number, Number]]
D64Vec3 = Union[glm.dmvec3, glm.dvec3]
D64Vector4 = Union[glm.dmvec4, glm.dvec4, Tuple[Number, Number, Number, Number]]
D64Vec4 = Union[glm.dmvec4, glm.dvec4]
D64VectorAny = Union[glm.dmvec2, glm.dmvec3, glm.dmvec4, glm.dvec1, glm.dvec2, glm.dvec3, glm.dvec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
D64VecAny = Union[glm.dmvec2, glm.dmvec3, glm.dmvec4, glm.dvec1, glm.dvec2, glm.dvec3, glm.dvec4]
DAnyVector1 = Union[glm.dvec1, Tuple[Number]]
DAnyVec1 = glm.dvec1
DAnyVector2 = Union[glm.dmvec2, glm.dvec2, Tuple[Number, Number]]
DAnyVec2 = Union[glm.dmvec2, glm.dvec2]
DAnyVector3 = Union[glm.dmvec3, glm.dvec3, Tuple[Number, Number, Number]]
DAnyVec3 = Union[glm.dmvec3, glm.dvec3]
DAnyVector4 = Union[glm.dmvec4, glm.dvec4, Tuple[Number, Number, Number, Number]]
DAnyVec4 = Union[glm.dmvec4, glm.dvec4]
DAnyVectorAny = Union[glm.dmvec2, glm.dmvec3, glm.dmvec4, glm.dvec1, glm.dvec2, glm.dvec3, glm.dvec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
DAnyVecAny = Union[glm.dmvec2, glm.dmvec3, glm.dmvec4, glm.dvec1, glm.dvec2, glm.dvec3, glm.dvec4]
F32Vector1 = Union[glm.vec1, Tuple[Number]]
F32Vec1 = glm.vec1
F32Vector2 = Union[glm.mvec2, glm.vec2, Tuple[Number, Number]]
F32Vec2 = Union[glm.mvec2, glm.vec2]
F32Vector3 = Union[glm.mvec3, glm.vec3, Tuple[Number, Number, Number]]
F32Vec3 = Union[glm.mvec3, glm.vec3]
F32Vector4 = Union[glm.mvec4, glm.vec4, Tuple[Number, Number, Number, Number]]
F32Vec4 = Union[glm.mvec4, glm.vec4]
F32VectorAny = Union[glm.mvec2, glm.mvec3, glm.mvec4, glm.vec1, glm.vec2, glm.vec3, glm.vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
F32VecAny = Union[glm.mvec2, glm.mvec3, glm.mvec4, glm.vec1, glm.vec2, glm.vec3, glm.vec4]
FAnyVector1 = Union[glm.vec1, Tuple[Number]]
FAnyVec1 = glm.vec1
FAnyVector2 = Union[glm.mvec2, glm.vec2, Tuple[Number, Number]]
FAnyVec2 = Union[glm.mvec2, glm.vec2]
FAnyVector3 = Union[glm.mvec3, glm.vec3, Tuple[Number, Number, Number]]
FAnyVec3 = Union[glm.mvec3, glm.vec3]
FAnyVector4 = Union[glm.mvec4, glm.vec4, Tuple[Number, Number, Number, Number]]
FAnyVec4 = Union[glm.mvec4, glm.vec4]
FAnyVectorAny = Union[glm.mvec2, glm.mvec3, glm.mvec4, glm.vec1, glm.vec2, glm.vec3, glm.vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
FAnyVecAny = Union[glm.mvec2, glm.mvec3, glm.mvec4, glm.vec1, glm.vec2, glm.vec3, glm.vec4]
I8Vector1 = Union[glm.i8vec1, Tuple[Number]]
I8Vec1 = glm.i8vec1
I8Vector2 = Union[glm.i8vec2, Tuple[Number, Number]]
I8Vec2 = glm.i8vec2
I8Vector3 = Union[glm.i8vec3, Tuple[Number, Number, Number]]
I8Vec3 = glm.i8vec3
I8Vector4 = Union[glm.i8vec4, Tuple[Number, Number, Number, Number]]
I8Vec4 = glm.i8vec4
I8VectorAny = Union[glm.i8vec1, glm.i8vec2, glm.i8vec3, glm.i8vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
I8VecAny = Union[glm.i8vec1, glm.i8vec2, glm.i8vec3, glm.i8vec4]
I16Vector1 = Union[glm.i16vec1, Tuple[Number]]
I16Vec1 = glm.i16vec1
I16Vector2 = Union[glm.i16vec2, Tuple[Number, Number]]
I16Vec2 = glm.i16vec2
I16Vector3 = Union[glm.i16vec3, Tuple[Number, Number, Number]]
I16Vec3 = glm.i16vec3
I16Vector4 = Union[glm.i16vec4, Tuple[Number, Number, Number, Number]]
I16Vec4 = glm.i16vec4
I16VectorAny = Union[glm.i16vec1, glm.i16vec2, glm.i16vec3, glm.i16vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
I16VecAny = Union[glm.i16vec1, glm.i16vec2, glm.i16vec3, glm.i16vec4]
I32Vector1 = Union[glm.ivec1, Tuple[Number]]
I32Vec1 = glm.ivec1
I32Vector2 = Union[glm.imvec2, glm.ivec2, Tuple[Number, Number]]
I32Vec2 = Union[glm.imvec2, glm.ivec2]
I32Vector3 = Union[glm.imvec3, glm.ivec3, Tuple[Number, Number, Number]]
I32Vec3 = Union[glm.imvec3, glm.ivec3]
I32Vector4 = Union[glm.imvec4, glm.ivec4, Tuple[Number, Number, Number, Number]]
I32Vec4 = Union[glm.imvec4, glm.ivec4]
I32VectorAny = Union[glm.imvec2, glm.imvec3, glm.imvec4, glm.ivec1, glm.ivec2, glm.ivec3, glm.ivec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
I32VecAny = Union[glm.imvec2, glm.imvec3, glm.imvec4, glm.ivec1, glm.ivec2, glm.ivec3, glm.ivec4]
I64Vector1 = Union[glm.i64vec1, Tuple[Number]]
I64Vec1 = glm.i64vec1
I64Vector2 = Union[glm.i64vec2, Tuple[Number, Number]]
I64Vec2 = glm.i64vec2
I64Vector3 = Union[glm.i64vec3, Tuple[Number, Number, Number]]
I64Vec3 = glm.i64vec3
I64Vector4 = Union[glm.i64vec4, Tuple[Number, Number, Number, Number]]
I64Vec4 = glm.i64vec4
I64VectorAny = Union[glm.i64vec1, glm.i64vec2, glm.i64vec3, glm.i64vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
I64VecAny = Union[glm.i64vec1, glm.i64vec2, glm.i64vec3, glm.i64vec4]
IAnyVector1 = Union[glm.i16vec1, glm.i64vec1, glm.i8vec1, glm.ivec1, Tuple[Number]]
IAnyVec1 = Union[glm.i16vec1, glm.i64vec1, glm.i8vec1, glm.ivec1]
IAnyVector2 = Union[glm.i16vec2, glm.i64vec2, glm.i8vec2, glm.imvec2, glm.ivec2, Tuple[Number, Number]]
IAnyVec2 = Union[glm.i16vec2, glm.i64vec2, glm.i8vec2, glm.imvec2, glm.ivec2]
IAnyVector3 = Union[glm.i16vec3, glm.i64vec3, glm.i8vec3, glm.imvec3, glm.ivec3, Tuple[Number, Number, Number]]
IAnyVec3 = Union[glm.i16vec3, glm.i64vec3, glm.i8vec3, glm.imvec3, glm.ivec3]
IAnyVector4 = Union[glm.i16vec4, glm.i64vec4, glm.i8vec4, glm.imvec4, glm.ivec4, Tuple[Number, Number, Number, Number]]
IAnyVec4 = Union[glm.i16vec4, glm.i64vec4, glm.i8vec4, glm.imvec4, glm.ivec4]
IAnyVectorAny = Union[glm.i16vec1, glm.i16vec2, glm.i16vec3, glm.i16vec4, glm.i64vec1, glm.i64vec2, glm.i64vec3, glm.i64vec4, glm.i8vec1, glm.i8vec2, glm.i8vec3, glm.i8vec4, glm.imvec2, glm.imvec3, glm.imvec4, glm.ivec1, glm.ivec2, glm.ivec3, glm.ivec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
IAnyVecAny = Union[glm.i16vec1, glm.i16vec2, glm.i16vec3, glm.i16vec4, glm.i64vec1, glm.i64vec2, glm.i64vec3, glm.i64vec4, glm.i8vec1, glm.i8vec2, glm.i8vec3, glm.i8vec4, glm.imvec2, glm.imvec3, glm.imvec4, glm.ivec1, glm.ivec2, glm.ivec3, glm.ivec4]
U8Vector1 = Union[glm.u8vec1, Tuple[Number]]
U8Vec1 = glm.u8vec1
U8Vector2 = Union[glm.u8vec2, Tuple[Number, Number]]
U8Vec2 = glm.u8vec2
U8Vector3 = Union[glm.u8vec3, Tuple[Number, Number, Number]]
U8Vec3 = glm.u8vec3
U8Vector4 = Union[glm.u8vec4, Tuple[Number, Number, Number, Number]]
U8Vec4 = glm.u8vec4
U8VectorAny = Union[glm.u8vec1, glm.u8vec2, glm.u8vec3, glm.u8vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
U8VecAny = Union[glm.u8vec1, glm.u8vec2, glm.u8vec3, glm.u8vec4]
U16Vector1 = Union[glm.u16vec1, Tuple[Number]]
U16Vec1 = glm.u16vec1
U16Vector2 = Union[glm.u16vec2, Tuple[Number, Number]]
U16Vec2 = glm.u16vec2
U16Vector3 = Union[glm.u16vec3, Tuple[Number, Number, Number]]
U16Vec3 = glm.u16vec3
U16Vector4 = Union[glm.u16vec4, Tuple[Number, Number, Number, Number]]
U16Vec4 = glm.u16vec4
U16VectorAny = Union[glm.u16vec1, glm.u16vec2, glm.u16vec3, glm.u16vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
U16VecAny = Union[glm.u16vec1, glm.u16vec2, glm.u16vec3, glm.u16vec4]
U32Vector1 = Union[glm.uvec1, Tuple[Number]]
U32Vec1 = glm.uvec1
U32Vector2 = Union[glm.umvec2, glm.uvec2, Tuple[Number, Number]]
U32Vec2 = Union[glm.umvec2, glm.uvec2]
U32Vector3 = Union[glm.umvec3, glm.uvec3, Tuple[Number, Number, Number]]
U32Vec3 = Union[glm.umvec3, glm.uvec3]
U32Vector4 = Union[glm.umvec4, glm.uvec4, Tuple[Number, Number, Number, Number]]
U32Vec4 = Union[glm.umvec4, glm.uvec4]
U32VectorAny = Union[glm.umvec2, glm.umvec3, glm.umvec4, glm.uvec1, glm.uvec2, glm.uvec3, glm.uvec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
U32VecAny = Union[glm.umvec2, glm.umvec3, glm.umvec4, glm.uvec1, glm.uvec2, glm.uvec3, glm.uvec4]
U64Vector1 = Union[glm.u64vec1, Tuple[Number]]
U64Vec1 = glm.u64vec1
U64Vector2 = Union[glm.u64vec2, Tuple[Number, Number]]
U64Vec2 = glm.u64vec2
U64Vector3 = Union[glm.u64vec3, Tuple[Number, Number, Number]]
U64Vec3 = glm.u64vec3
U64Vector4 = Union[glm.u64vec4, Tuple[Number, Number, Number, Number]]
U64Vec4 = glm.u64vec4
U64VectorAny = Union[glm.u64vec1, glm.u64vec2, glm.u64vec3, glm.u64vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
U64VecAny = Union[glm.u64vec1, glm.u64vec2, glm.u64vec3, glm.u64vec4]
UAnyVector1 = Union[glm.u16vec1, glm.u64vec1, glm.u8vec1, glm.uvec1, Tuple[Number]]
UAnyVec1 = Union[glm.u16vec1, glm.u64vec1, glm.u8vec1, glm.uvec1]
UAnyVector2 = Union[glm.u16vec2, glm.u64vec2, glm.u8vec2, glm.umvec2, glm.uvec2, Tuple[Number, Number]]
UAnyVec2 = Union[glm.u16vec2, glm.u64vec2, glm.u8vec2, glm.umvec2, glm.uvec2]
UAnyVector3 = Union[glm.u16vec3, glm.u64vec3, glm.u8vec3, glm.umvec3, glm.uvec3, Tuple[Number, Number, Number]]
UAnyVec3 = Union[glm.u16vec3, glm.u64vec3, glm.u8vec3, glm.umvec3, glm.uvec3]
UAnyVector4 = Union[glm.u16vec4, glm.u64vec4, glm.u8vec4, glm.umvec4, glm.uvec4, Tuple[Number, Number, Number, Number]]
UAnyVec4 = Union[glm.u16vec4, glm.u64vec4, glm.u8vec4, glm.umvec4, glm.uvec4]
UAnyVectorAny = Union[glm.u16vec1, glm.u16vec2, glm.u16vec3, glm.u16vec4, glm.u64vec1, glm.u64vec2, glm.u64vec3, glm.u64vec4, glm.u8vec1, glm.u8vec2, glm.u8vec3, glm.u8vec4, glm.umvec2, glm.umvec3, glm.umvec4, glm.uvec1, glm.uvec2, glm.uvec3, glm.uvec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
UAnyVecAny = Union[glm.u16vec1, glm.u16vec2, glm.u16vec3, glm.u16vec4, glm.u64vec1, glm.u64vec2, glm.u64vec3, glm.u64vec4, glm.u8vec1, glm.u8vec2, glm.u8vec3, glm.u8vec4, glm.umvec2, glm.umvec3, glm.umvec4, glm.uvec1, glm.uvec2, glm.uvec3, glm.uvec4]
Any8Vector1 = Union[glm.bvec1, glm.i8vec1, glm.u8vec1, Tuple[Number]]
Any8Vec1 = Union[glm.bvec1, glm.i8vec1, glm.u8vec1]
Any8Vector2 = Union[glm.bvec2, glm.i8vec2, glm.u8vec2, Tuple[Number, Number]]
Any8Vec2 = Union[glm.bvec2, glm.i8vec2, glm.u8vec2]
Any8Vector3 = Union[glm.bvec3, glm.i8vec3, glm.u8vec3, Tuple[Number, Number, Number]]
Any8Vec3 = Union[glm.bvec3, glm.i8vec3, glm.u8vec3]
Any8Vector4 = Union[glm.bvec4, glm.i8vec4, glm.u8vec4, Tuple[Number, Number, Number, Number]]
Any8Vec4 = Union[glm.bvec4, glm.i8vec4, glm.u8vec4]
Any8VectorAny = Union[glm.bvec1, glm.bvec2, glm.bvec3, glm.bvec4, glm.i8vec1, glm.i8vec2, glm.i8vec3, glm.i8vec4, glm.u8vec1, glm.u8vec2, glm.u8vec3, glm.u8vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
Any8VecAny = Union[glm.bvec1, glm.bvec2, glm.bvec3, glm.bvec4, glm.i8vec1, glm.i8vec2, glm.i8vec3, glm.i8vec4, glm.u8vec1, glm.u8vec2, glm.u8vec3, glm.u8vec4]
Any16Vector1 = Union[glm.i16vec1, glm.u16vec1, Tuple[Number]]
Any16Vec1 = Union[glm.i16vec1, glm.u16vec1]
Any16Vector2 = Union[glm.i16vec2, glm.u16vec2, Tuple[Number, Number]]
Any16Vec2 = Union[glm.i16vec2, glm.u16vec2]
Any16Vector3 = Union[glm.i16vec3, glm.u16vec3, Tuple[Number, Number, Number]]
Any16Vec3 = Union[glm.i16vec3, glm.u16vec3]
Any16Vector4 = Union[glm.i16vec4, glm.u16vec4, Tuple[Number, Number, Number, Number]]
Any16Vec4 = Union[glm.i16vec4, glm.u16vec4]
Any16VectorAny = Union[glm.i16vec1, glm.i16vec2, glm.i16vec3, glm.i16vec4, glm.u16vec1, glm.u16vec2, glm.u16vec3, glm.u16vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
Any16VecAny = Union[glm.i16vec1, glm.i16vec2, glm.i16vec3, glm.i16vec4, glm.u16vec1, glm.u16vec2, glm.u16vec3, glm.u16vec4]
Any32Vector1 = Union[glm.ivec1, glm.uvec1, glm.vec1, Tuple[Number]]
Any32Vec1 = Union[glm.ivec1, glm.uvec1, glm.vec1]
Any32Vector2 = Union[glm.imvec2, glm.ivec2, glm.mvec2, glm.umvec2, glm.uvec2, glm.vec2, Tuple[Number, Number]]
Any32Vec2 = Union[glm.imvec2, glm.ivec2, glm.mvec2, glm.umvec2, glm.uvec2, glm.vec2]
Any32Vector3 = Union[glm.imvec3, glm.ivec3, glm.mvec3, glm.umvec3, glm.uvec3, glm.vec3, Tuple[Number, Number, Number]]
Any32Vec3 = Union[glm.imvec3, glm.ivec3, glm.mvec3, glm.umvec3, glm.uvec3, glm.vec3]
Any32Vector4 = Union[glm.imvec4, glm.ivec4, glm.mvec4, glm.umvec4, glm.uvec4, glm.vec4, Tuple[Number, Number, Number, Number]]
Any32Vec4 = Union[glm.imvec4, glm.ivec4, glm.mvec4, glm.umvec4, glm.uvec4, glm.vec4]
Any32VectorAny = Union[glm.imvec2, glm.imvec3, glm.imvec4, glm.ivec1, glm.ivec2, glm.ivec3, glm.ivec4, glm.mvec2, glm.mvec3, glm.mvec4, glm.umvec2, glm.umvec3, glm.umvec4, glm.uvec1, glm.uvec2, glm.uvec3, glm.uvec4, glm.vec1, glm.vec2, glm.vec3, glm.vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
Any32VecAny = Union[glm.imvec2, glm.imvec3, glm.imvec4, glm.ivec1, glm.ivec2, glm.ivec3, glm.ivec4, glm.mvec2, glm.mvec3, glm.mvec4, glm.umvec2, glm.umvec3, glm.umvec4, glm.uvec1, glm.uvec2, glm.uvec3, glm.uvec4, glm.vec1, glm.vec2, glm.vec3, glm.vec4]
Any64Vector1 = Union[glm.dvec1, glm.i64vec1, glm.u64vec1, Tuple[Number]]
Any64Vec1 = Union[glm.dvec1, glm.i64vec1, glm.u64vec1]
Any64Vector2 = Union[glm.dmvec2, glm.dvec2, glm.i64vec2, glm.u64vec2, Tuple[Number, Number]]
Any64Vec2 = Union[glm.dmvec2, glm.dvec2, glm.i64vec2, glm.u64vec2]
Any64Vector3 = Union[glm.dmvec3, glm.dvec3, glm.i64vec3, glm.u64vec3, Tuple[Number, Number, Number]]
Any64Vec3 = Union[glm.dmvec3, glm.dvec3, glm.i64vec3, glm.u64vec3]
Any64Vector4 = Union[glm.dmvec4, glm.dvec4, glm.i64vec4, glm.u64vec4, Tuple[Number, Number, Number, Number]]
Any64Vec4 = Union[glm.dmvec4, glm.dvec4, glm.i64vec4, glm.u64vec4]
Any64VectorAny = Union[glm.dmvec2, glm.dmvec3, glm.dmvec4, glm.dvec1, glm.dvec2, glm.dvec3, glm.dvec4, glm.i64vec1, glm.i64vec2, glm.i64vec3, glm.i64vec4, glm.u64vec1, glm.u64vec2, glm.u64vec3, glm.u64vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
Any64VecAny = Union[glm.dmvec2, glm.dmvec3, glm.dmvec4, glm.dvec1, glm.dvec2, glm.dvec3, glm.dvec4, glm.i64vec1, glm.i64vec2, glm.i64vec3, glm.i64vec4, glm.u64vec1, glm.u64vec2, glm.u64vec3, glm.u64vec4]
AnyAnyVector1 = Union[glm.bvec1, glm.dvec1, glm.i16vec1, glm.i64vec1, glm.i8vec1, glm.ivec1, glm.u16vec1, glm.u64vec1, glm.u8vec1, glm.uvec1, glm.vec1, Tuple[Number]]
AnyAnyVec1 = Union[glm.bvec1, glm.dvec1, glm.i16vec1, glm.i64vec1, glm.i8vec1, glm.ivec1, glm.u16vec1, glm.u64vec1, glm.u8vec1, glm.uvec1, glm.vec1]
AnyAnyVector2 = Union[glm.bvec2, glm.dmvec2, glm.dvec2, glm.i16vec2, glm.i64vec2, glm.i8vec2, glm.imvec2, glm.ivec2, glm.mvec2, glm.u16vec2, glm.u64vec2, glm.u8vec2, glm.umvec2, glm.uvec2, glm.vec2, Tuple[Number, Number]]
AnyAnyVec2 = Union[glm.bvec2, glm.dmvec2, glm.dvec2, glm.i16vec2, glm.i64vec2, glm.i8vec2, glm.imvec2, glm.ivec2, glm.mvec2, glm.u16vec2, glm.u64vec2, glm.u8vec2, glm.umvec2, glm.uvec2, glm.vec2]
AnyAnyVector3 = Union[glm.bvec3, glm.dmvec3, glm.dvec3, glm.i16vec3, glm.i64vec3, glm.i8vec3, glm.imvec3, glm.ivec3, glm.mvec3, glm.u16vec3, glm.u64vec3, glm.u8vec3, glm.umvec3, glm.uvec3, glm.vec3, Tuple[Number, Number, Number]]
AnyAnyVec3 = Union[glm.bvec3, glm.dmvec3, glm.dvec3, glm.i16vec3, glm.i64vec3, glm.i8vec3, glm.imvec3, glm.ivec3, glm.mvec3, glm.u16vec3, glm.u64vec3, glm.u8vec3, glm.umvec3, glm.uvec3, glm.vec3]
AnyAnyVector4 = Union[glm.bvec4, glm.dmvec4, glm.dvec4, glm.i16vec4, glm.i64vec4, glm.i8vec4, glm.imvec4, glm.ivec4, glm.mvec4, glm.u16vec4, glm.u64vec4, glm.u8vec4, glm.umvec4, glm.uvec4, glm.vec4, Tuple[Number, Number, Number, Number]]
AnyAnyVec4 = Union[glm.bvec4, glm.dmvec4, glm.dvec4, glm.i16vec4, glm.i64vec4, glm.i8vec4, glm.imvec4, glm.ivec4, glm.mvec4, glm.u16vec4, glm.u64vec4, glm.u8vec4, glm.umvec4, glm.uvec4, glm.vec4]
AnyAnyVectorAny = Union[glm.bvec1, glm.bvec2, glm.bvec3, glm.bvec4, glm.dmvec2, glm.dmvec3, glm.dmvec4, glm.dvec1, glm.dvec2, glm.dvec3, glm.dvec4, glm.i16vec1, glm.i16vec2, glm.i16vec3, glm.i16vec4, glm.i64vec1, glm.i64vec2, glm.i64vec3, glm.i64vec4, glm.i8vec1, glm.i8vec2, glm.i8vec3, glm.i8vec4, glm.imvec2, glm.imvec3, glm.imvec4, glm.ivec1, glm.ivec2, glm.ivec3, glm.ivec4, glm.mvec2, glm.mvec3, glm.mvec4, glm.u16vec1, glm.u16vec2, glm.u16vec3, glm.u16vec4, glm.u64vec1, glm.u64vec2, glm.u64vec3, glm.u64vec4, glm.u8vec1, glm.u8vec2, glm.u8vec3, glm.u8vec4, glm.umvec2, glm.umvec3, glm.umvec4, glm.uvec1, glm.uvec2, glm.uvec3, glm.uvec4, glm.vec1, glm.vec2, glm.vec3, glm.vec4, Tuple[Number], Tuple[Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number, Number]]
AnyAnyVecAny = Union[glm.bvec1, glm.bvec2, glm.bvec3, glm.bvec4, glm.dmvec2, glm.dmvec3, glm.dmvec4, glm.dvec1, glm.dvec2, glm.dvec3, glm.dvec4, glm.i16vec1, glm.i16vec2, glm.i16vec3, glm.i16vec4, glm.i64vec1, glm.i64vec2, glm.i64vec3, glm.i64vec4, glm.i8vec1, glm.i8vec2, glm.i8vec3, glm.i8vec4, glm.imvec2, glm.imvec3, glm.imvec4, glm.ivec1, glm.ivec2, glm.ivec3, glm.ivec4, glm.mvec2, glm.mvec3, glm.mvec4, glm.u16vec1, glm.u16vec2, glm.u16vec3, glm.u16vec4, glm.u64vec1, glm.u64vec2, glm.u64vec3, glm.u64vec4, glm.u8vec1, glm.u8vec2, glm.u8vec3, glm.u8vec4, glm.umvec2, glm.umvec3, glm.umvec4, glm.uvec1, glm.uvec2, glm.uvec3, glm.uvec4, glm.vec1, glm.vec2, glm.vec3, glm.vec4]
FDAnyVectorAny = Union[FAnyVectorAny, DAnyVectorAny]
FDAnyVector1 = Union[FAnyVector1, DAnyVector1]
FDAnyVector2 = Union[FAnyVector2, DAnyVector2]
FDAnyVector3 = Union[FAnyVector3, DAnyVector3]
FDAnyVector4 = Union[FAnyVector4, DAnyVector4]
IUAnyVector1 = Union[IAnyVector1, UAnyVector1]
IUAnyVector2 = Union[IAnyVector2, UAnyVector2]
IUAnyVector3 = Union[IAnyVector3, UAnyVector3]
IUAnyVector4 = Union[IAnyVector4, UAnyVector4]
D64Matrix2x2 = Union[glm.dmat2x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]]]
D64Mat2x2 = glm.dmat2x2
D64Matrix2x3 = Union[glm.dmat2x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
D64Mat2x3 = glm.dmat2x3
D64Matrix2x4 = Union[glm.dmat2x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
D64Mat2x4 = glm.dmat2x4
D64Matrix2xAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
D64Mat2xAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4]
D64Matrix3x2 = Union[glm.dmat3x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
D64Mat3x2 = glm.dmat3x2
D64Matrix3x3 = Union[glm.dmat3x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
D64Mat3x3 = glm.dmat3x3
D64Matrix3x4 = Union[glm.dmat3x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
D64Mat3x4 = glm.dmat3x4
D64Matrix3xAny = Union[glm.dmat3x2, glm.dmat3x3, glm.dmat3x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
D64Mat3xAny = Union[glm.dmat3x2, glm.dmat3x3, glm.dmat3x4]
D64Matrix4x2 = Union[glm.dmat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
D64Mat4x2 = glm.dmat4x2
D64Matrix4x3 = Union[glm.dmat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
D64Mat4x3 = glm.dmat4x3
D64Matrix4x4 = Union[glm.dmat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
D64Mat4x4 = glm.dmat4x4
D64Matrix4xAny = Union[glm.dmat4x2, glm.dmat4x3, glm.dmat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
D64Mat4xAny = Union[glm.dmat4x2, glm.dmat4x3, glm.dmat4x4]
D64MatrixAnyx2 = Union[glm.dmat2x2, glm.dmat3x2, glm.dmat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
D64MatAnyx2 = Union[glm.dmat2x2, glm.dmat3x2, glm.dmat4x2]
D64MatrixAnyx3 = Union[glm.dmat2x3, glm.dmat3x3, glm.dmat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
D64MatAnyx3 = Union[glm.dmat2x3, glm.dmat3x3, glm.dmat4x3]
D64MatrixAnyx4 = Union[glm.dmat2x4, glm.dmat3x4, glm.dmat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
D64MatAnyx4 = Union[glm.dmat2x4, glm.dmat3x4, glm.dmat4x4]
D64MatrixAnyxAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4, glm.dmat3x2, glm.dmat3x3, glm.dmat3x4, glm.dmat4x2, glm.dmat4x3, glm.dmat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
D64MatAnyxAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4, glm.dmat3x2, glm.dmat3x3, glm.dmat3x4, glm.dmat4x2, glm.dmat4x3, glm.dmat4x4]
DAnyMatrix2x2 = Union[glm.dmat2x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]]]
DAnyMat2x2 = glm.dmat2x2
DAnyMatrix2x3 = Union[glm.dmat2x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
DAnyMat2x3 = glm.dmat2x3
DAnyMatrix2x4 = Union[glm.dmat2x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
DAnyMat2x4 = glm.dmat2x4
DAnyMatrix2xAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
DAnyMat2xAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4]
DAnyMatrix3x2 = Union[glm.dmat3x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
DAnyMat3x2 = glm.dmat3x2
DAnyMatrix3x3 = Union[glm.dmat3x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
DAnyMat3x3 = glm.dmat3x3
DAnyMatrix3x4 = Union[glm.dmat3x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
DAnyMat3x4 = glm.dmat3x4
DAnyMatrix3xAny = Union[glm.dmat3x2, glm.dmat3x3, glm.dmat3x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
DAnyMat3xAny = Union[glm.dmat3x2, glm.dmat3x3, glm.dmat3x4]
DAnyMatrix4x2 = Union[glm.dmat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
DAnyMat4x2 = glm.dmat4x2
DAnyMatrix4x3 = Union[glm.dmat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
DAnyMat4x3 = glm.dmat4x3
DAnyMatrix4x4 = Union[glm.dmat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
DAnyMat4x4 = glm.dmat4x4
DAnyMatrix4xAny = Union[glm.dmat4x2, glm.dmat4x3, glm.dmat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
DAnyMat4xAny = Union[glm.dmat4x2, glm.dmat4x3, glm.dmat4x4]
DAnyMatrixAnyx2 = Union[glm.dmat2x2, glm.dmat3x2, glm.dmat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
DAnyMatAnyx2 = Union[glm.dmat2x2, glm.dmat3x2, glm.dmat4x2]
DAnyMatrixAnyx3 = Union[glm.dmat2x3, glm.dmat3x3, glm.dmat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
DAnyMatAnyx3 = Union[glm.dmat2x3, glm.dmat3x3, glm.dmat4x3]
DAnyMatrixAnyx4 = Union[glm.dmat2x4, glm.dmat3x4, glm.dmat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
DAnyMatAnyx4 = Union[glm.dmat2x4, glm.dmat3x4, glm.dmat4x4]
DAnyMatrixAnyxAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4, glm.dmat3x2, glm.dmat3x3, glm.dmat3x4, glm.dmat4x2, glm.dmat4x3, glm.dmat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
DAnyMatAnyxAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4, glm.dmat3x2, glm.dmat3x3, glm.dmat3x4, glm.dmat4x2, glm.dmat4x3, glm.dmat4x4]
F32Matrix2x2 = Union[glm.mat2x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]]]
F32Mat2x2 = glm.mat2x2
F32Matrix2x3 = Union[glm.mat2x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
F32Mat2x3 = glm.mat2x3
F32Matrix2x4 = Union[glm.mat2x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
F32Mat2x4 = glm.mat2x4
F32Matrix2xAny = Union[glm.mat2x2, glm.mat2x3, glm.mat2x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
F32Mat2xAny = Union[glm.mat2x2, glm.mat2x3, glm.mat2x4]
F32Matrix3x2 = Union[glm.mat3x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
F32Mat3x2 = glm.mat3x2
F32Matrix3x3 = Union[glm.mat3x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
F32Mat3x3 = glm.mat3x3
F32Matrix3x4 = Union[glm.mat3x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
F32Mat3x4 = glm.mat3x4
F32Matrix3xAny = Union[glm.mat3x2, glm.mat3x3, glm.mat3x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
F32Mat3xAny = Union[glm.mat3x2, glm.mat3x3, glm.mat3x4]
F32Matrix4x2 = Union[glm.mat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
F32Mat4x2 = glm.mat4x2
F32Matrix4x3 = Union[glm.mat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
F32Mat4x3 = glm.mat4x3
F32Matrix4x4 = Union[glm.mat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
F32Mat4x4 = glm.mat4x4
F32Matrix4xAny = Union[glm.mat4x2, glm.mat4x3, glm.mat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
F32Mat4xAny = Union[glm.mat4x2, glm.mat4x3, glm.mat4x4]
F32MatrixAnyx2 = Union[glm.mat2x2, glm.mat3x2, glm.mat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
F32MatAnyx2 = Union[glm.mat2x2, glm.mat3x2, glm.mat4x2]
F32MatrixAnyx3 = Union[glm.mat2x3, glm.mat3x3, glm.mat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
F32MatAnyx3 = Union[glm.mat2x3, glm.mat3x3, glm.mat4x3]
F32MatrixAnyx4 = Union[glm.mat2x4, glm.mat3x4, glm.mat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
F32MatAnyx4 = Union[glm.mat2x4, glm.mat3x4, glm.mat4x4]
F32MatrixAnyxAny = Union[glm.mat2x2, glm.mat2x3, glm.mat2x4, glm.mat3x2, glm.mat3x3, glm.mat3x4, glm.mat4x2, glm.mat4x3, glm.mat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
F32MatAnyxAny = Union[glm.mat2x2, glm.mat2x3, glm.mat2x4, glm.mat3x2, glm.mat3x3, glm.mat3x4, glm.mat4x2, glm.mat4x3, glm.mat4x4]
FAnyMatrix2x2 = Union[glm.mat2x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]]]
FAnyMat2x2 = glm.mat2x2
FAnyMatrix2x3 = Union[glm.mat2x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
FAnyMat2x3 = glm.mat2x3
FAnyMatrix2x4 = Union[glm.mat2x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
FAnyMat2x4 = glm.mat2x4
FAnyMatrix2xAny = Union[glm.mat2x2, glm.mat2x3, glm.mat2x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
FAnyMat2xAny = Union[glm.mat2x2, glm.mat2x3, glm.mat2x4]
FAnyMatrix3x2 = Union[glm.mat3x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
FAnyMat3x2 = glm.mat3x2
FAnyMatrix3x3 = Union[glm.mat3x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
FAnyMat3x3 = glm.mat3x3
FAnyMatrix3x4 = Union[glm.mat3x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
FAnyMat3x4 = glm.mat3x4
FAnyMatrix3xAny = Union[glm.mat3x2, glm.mat3x3, glm.mat3x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
FAnyMat3xAny = Union[glm.mat3x2, glm.mat3x3, glm.mat3x4]
FAnyMatrix4x2 = Union[glm.mat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
FAnyMat4x2 = glm.mat4x2
FAnyMatrix4x3 = Union[glm.mat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
FAnyMat4x3 = glm.mat4x3
FAnyMatrix4x4 = Union[glm.mat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
FAnyMat4x4 = glm.mat4x4
FAnyMatrix4xAny = Union[glm.mat4x2, glm.mat4x3, glm.mat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
FAnyMat4xAny = Union[glm.mat4x2, glm.mat4x3, glm.mat4x4]
FAnyMatrixAnyx2 = Union[glm.mat2x2, glm.mat3x2, glm.mat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
FAnyMatAnyx2 = Union[glm.mat2x2, glm.mat3x2, glm.mat4x2]
FAnyMatrixAnyx3 = Union[glm.mat2x3, glm.mat3x3, glm.mat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
FAnyMatAnyx3 = Union[glm.mat2x3, glm.mat3x3, glm.mat4x3]
FAnyMatrixAnyx4 = Union[glm.mat2x4, glm.mat3x4, glm.mat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
FAnyMatAnyx4 = Union[glm.mat2x4, glm.mat3x4, glm.mat4x4]
FAnyMatrixAnyxAny = Union[glm.mat2x2, glm.mat2x3, glm.mat2x4, glm.mat3x2, glm.mat3x3, glm.mat3x4, glm.mat4x2, glm.mat4x3, glm.mat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
FAnyMatAnyxAny = Union[glm.mat2x2, glm.mat2x3, glm.mat2x4, glm.mat3x2, glm.mat3x3, glm.mat3x4, glm.mat4x2, glm.mat4x3, glm.mat4x4]
I32Matrix2x2 = Union[glm.imat2x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]]]
I32Mat2x2 = glm.imat2x2
I32Matrix2x3 = Union[glm.imat2x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
I32Mat2x3 = glm.imat2x3
I32Matrix2x4 = Union[glm.imat2x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
I32Mat2x4 = glm.imat2x4
I32Matrix2xAny = Union[glm.imat2x2, glm.imat2x3, glm.imat2x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
I32Mat2xAny = Union[glm.imat2x2, glm.imat2x3, glm.imat2x4]
I32Matrix3x2 = Union[glm.imat3x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
I32Mat3x2 = glm.imat3x2
I32Matrix3x3 = Union[glm.imat3x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
I32Mat3x3 = glm.imat3x3
I32Matrix3x4 = Union[glm.imat3x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
I32Mat3x4 = glm.imat3x4
I32Matrix3xAny = Union[glm.imat3x2, glm.imat3x3, glm.imat3x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
I32Mat3xAny = Union[glm.imat3x2, glm.imat3x3, glm.imat3x4]
I32Matrix4x2 = Union[glm.imat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
I32Mat4x2 = glm.imat4x2
I32Matrix4x3 = Union[glm.imat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
I32Mat4x3 = glm.imat4x3
I32Matrix4x4 = Union[glm.imat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
I32Mat4x4 = glm.imat4x4
I32Matrix4xAny = Union[glm.imat4x2, glm.imat4x3, glm.imat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
I32Mat4xAny = Union[glm.imat4x2, glm.imat4x3, glm.imat4x4]
I32MatrixAnyx2 = Union[glm.imat2x2, glm.imat3x2, glm.imat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
I32MatAnyx2 = Union[glm.imat2x2, glm.imat3x2, glm.imat4x2]
I32MatrixAnyx3 = Union[glm.imat2x3, glm.imat3x3, glm.imat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
I32MatAnyx3 = Union[glm.imat2x3, glm.imat3x3, glm.imat4x3]
I32MatrixAnyx4 = Union[glm.imat2x4, glm.imat3x4, glm.imat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
I32MatAnyx4 = Union[glm.imat2x4, glm.imat3x4, glm.imat4x4]
I32MatrixAnyxAny = Union[glm.imat2x2, glm.imat2x3, glm.imat2x4, glm.imat3x2, glm.imat3x3, glm.imat3x4, glm.imat4x2, glm.imat4x3, glm.imat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
I32MatAnyxAny = Union[glm.imat2x2, glm.imat2x3, glm.imat2x4, glm.imat3x2, glm.imat3x3, glm.imat3x4, glm.imat4x2, glm.imat4x3, glm.imat4x4]
IAnyMatrix2x2 = Union[glm.imat2x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]]]
IAnyMat2x2 = glm.imat2x2
IAnyMatrix2x3 = Union[glm.imat2x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
IAnyMat2x3 = glm.imat2x3
IAnyMatrix2x4 = Union[glm.imat2x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
IAnyMat2x4 = glm.imat2x4
IAnyMatrix2xAny = Union[glm.imat2x2, glm.imat2x3, glm.imat2x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
IAnyMat2xAny = Union[glm.imat2x2, glm.imat2x3, glm.imat2x4]
IAnyMatrix3x2 = Union[glm.imat3x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
IAnyMat3x2 = glm.imat3x2
IAnyMatrix3x3 = Union[glm.imat3x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
IAnyMat3x3 = glm.imat3x3
IAnyMatrix3x4 = Union[glm.imat3x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
IAnyMat3x4 = glm.imat3x4
IAnyMatrix3xAny = Union[glm.imat3x2, glm.imat3x3, glm.imat3x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
IAnyMat3xAny = Union[glm.imat3x2, glm.imat3x3, glm.imat3x4]
IAnyMatrix4x2 = Union[glm.imat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
IAnyMat4x2 = glm.imat4x2
IAnyMatrix4x3 = Union[glm.imat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
IAnyMat4x3 = glm.imat4x3
IAnyMatrix4x4 = Union[glm.imat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
IAnyMat4x4 = glm.imat4x4
IAnyMatrix4xAny = Union[glm.imat4x2, glm.imat4x3, glm.imat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
IAnyMat4xAny = Union[glm.imat4x2, glm.imat4x3, glm.imat4x4]
IAnyMatrixAnyx2 = Union[glm.imat2x2, glm.imat3x2, glm.imat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
IAnyMatAnyx2 = Union[glm.imat2x2, glm.imat3x2, glm.imat4x2]
IAnyMatrixAnyx3 = Union[glm.imat2x3, glm.imat3x3, glm.imat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
IAnyMatAnyx3 = Union[glm.imat2x3, glm.imat3x3, glm.imat4x3]
IAnyMatrixAnyx4 = Union[glm.imat2x4, glm.imat3x4, glm.imat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
IAnyMatAnyx4 = Union[glm.imat2x4, glm.imat3x4, glm.imat4x4]
IAnyMatrixAnyxAny = Union[glm.imat2x2, glm.imat2x3, glm.imat2x4, glm.imat3x2, glm.imat3x3, glm.imat3x4, glm.imat4x2, glm.imat4x3, glm.imat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
IAnyMatAnyxAny = Union[glm.imat2x2, glm.imat2x3, glm.imat2x4, glm.imat3x2, glm.imat3x3, glm.imat3x4, glm.imat4x2, glm.imat4x3, glm.imat4x4]
U32Matrix2x2 = Union[glm.umat2x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]]]
U32Mat2x2 = glm.umat2x2
U32Matrix2x3 = Union[glm.umat2x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
U32Mat2x3 = glm.umat2x3
U32Matrix2x4 = Union[glm.umat2x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
U32Mat2x4 = glm.umat2x4
U32Matrix2xAny = Union[glm.umat2x2, glm.umat2x3, glm.umat2x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
U32Mat2xAny = Union[glm.umat2x2, glm.umat2x3, glm.umat2x4]
U32Matrix3x2 = Union[glm.umat3x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
U32Mat3x2 = glm.umat3x2
U32Matrix3x3 = Union[glm.umat3x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
U32Mat3x3 = glm.umat3x3
U32Matrix3x4 = Union[glm.umat3x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
U32Mat3x4 = glm.umat3x4
U32Matrix3xAny = Union[glm.umat3x2, glm.umat3x3, glm.umat3x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
U32Mat3xAny = Union[glm.umat3x2, glm.umat3x3, glm.umat3x4]
U32Matrix4x2 = Union[glm.umat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
U32Mat4x2 = glm.umat4x2
U32Matrix4x3 = Union[glm.umat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
U32Mat4x3 = glm.umat4x3
U32Matrix4x4 = Union[glm.umat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
U32Mat4x4 = glm.umat4x4
U32Matrix4xAny = Union[glm.umat4x2, glm.umat4x3, glm.umat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
U32Mat4xAny = Union[glm.umat4x2, glm.umat4x3, glm.umat4x4]
U32MatrixAnyx2 = Union[glm.umat2x2, glm.umat3x2, glm.umat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
U32MatAnyx2 = Union[glm.umat2x2, glm.umat3x2, glm.umat4x2]
U32MatrixAnyx3 = Union[glm.umat2x3, glm.umat3x3, glm.umat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
U32MatAnyx3 = Union[glm.umat2x3, glm.umat3x3, glm.umat4x3]
U32MatrixAnyx4 = Union[glm.umat2x4, glm.umat3x4, glm.umat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
U32MatAnyx4 = Union[glm.umat2x4, glm.umat3x4, glm.umat4x4]
U32MatrixAnyxAny = Union[glm.umat2x2, glm.umat2x3, glm.umat2x4, glm.umat3x2, glm.umat3x3, glm.umat3x4, glm.umat4x2, glm.umat4x3, glm.umat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
U32MatAnyxAny = Union[glm.umat2x2, glm.umat2x3, glm.umat2x4, glm.umat3x2, glm.umat3x3, glm.umat3x4, glm.umat4x2, glm.umat4x3, glm.umat4x4]
UAnyMatrix2x2 = Union[glm.umat2x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]]]
UAnyMat2x2 = glm.umat2x2
UAnyMatrix2x3 = Union[glm.umat2x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
UAnyMat2x3 = glm.umat2x3
UAnyMatrix2x4 = Union[glm.umat2x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
UAnyMat2x4 = glm.umat2x4
UAnyMatrix2xAny = Union[glm.umat2x2, glm.umat2x3, glm.umat2x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
UAnyMat2xAny = Union[glm.umat2x2, glm.umat2x3, glm.umat2x4]
UAnyMatrix3x2 = Union[glm.umat3x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
UAnyMat3x2 = glm.umat3x2
UAnyMatrix3x3 = Union[glm.umat3x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
UAnyMat3x3 = glm.umat3x3
UAnyMatrix3x4 = Union[glm.umat3x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
UAnyMat3x4 = glm.umat3x4
UAnyMatrix3xAny = Union[glm.umat3x2, glm.umat3x3, glm.umat3x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
UAnyMat3xAny = Union[glm.umat3x2, glm.umat3x3, glm.umat3x4]
UAnyMatrix4x2 = Union[glm.umat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
UAnyMat4x2 = glm.umat4x2
UAnyMatrix4x3 = Union[glm.umat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
UAnyMat4x3 = glm.umat4x3
UAnyMatrix4x4 = Union[glm.umat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
UAnyMat4x4 = glm.umat4x4
UAnyMatrix4xAny = Union[glm.umat4x2, glm.umat4x3, glm.umat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
UAnyMat4xAny = Union[glm.umat4x2, glm.umat4x3, glm.umat4x4]
UAnyMatrixAnyx2 = Union[glm.umat2x2, glm.umat3x2, glm.umat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
UAnyMatAnyx2 = Union[glm.umat2x2, glm.umat3x2, glm.umat4x2]
UAnyMatrixAnyx3 = Union[glm.umat2x3, glm.umat3x3, glm.umat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
UAnyMatAnyx3 = Union[glm.umat2x3, glm.umat3x3, glm.umat4x3]
UAnyMatrixAnyx4 = Union[glm.umat2x4, glm.umat3x4, glm.umat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
UAnyMatAnyx4 = Union[glm.umat2x4, glm.umat3x4, glm.umat4x4]
UAnyMatrixAnyxAny = Union[glm.umat2x2, glm.umat2x3, glm.umat2x4, glm.umat3x2, glm.umat3x3, glm.umat3x4, glm.umat4x2, glm.umat4x3, glm.umat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
UAnyMatAnyxAny = Union[glm.umat2x2, glm.umat2x3, glm.umat2x4, glm.umat3x2, glm.umat3x3, glm.umat3x4, glm.umat4x2, glm.umat4x3, glm.umat4x4]
AnyAnyMatrix2x2 = Union[glm.dmat2x2, glm.imat2x2, glm.mat2x2, glm.umat2x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]]]
AnyAnyMat2x2 = Union[glm.dmat2x2, glm.imat2x2, glm.mat2x2, glm.umat2x2]
AnyAnyMatrix2x3 = Union[glm.dmat2x3, glm.imat2x3, glm.mat2x3, glm.umat2x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
AnyAnyMat2x3 = Union[glm.dmat2x3, glm.imat2x3, glm.mat2x3, glm.umat2x3]
AnyAnyMatrix2x4 = Union[glm.dmat2x4, glm.imat2x4, glm.mat2x4, glm.umat2x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
AnyAnyMat2x4 = Union[glm.dmat2x4, glm.imat2x4, glm.mat2x4, glm.umat2x4]
AnyAnyMatrix2xAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4, glm.imat2x2, glm.imat2x3, glm.imat2x4, glm.mat2x2, glm.mat2x3, glm.mat2x4, glm.umat2x2, glm.umat2x3, glm.umat2x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
AnyAnyMat2xAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4, glm.imat2x2, glm.imat2x3, glm.imat2x4, glm.mat2x2, glm.mat2x3, glm.mat2x4, glm.umat2x2, glm.umat2x3, glm.umat2x4]
AnyAnyMatrix3x2 = Union[glm.dmat3x2, glm.imat3x2, glm.mat3x2, glm.umat3x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
AnyAnyMat3x2 = Union[glm.dmat3x2, glm.imat3x2, glm.mat3x2, glm.umat3x2]
AnyAnyMatrix3x3 = Union[glm.dmat3x3, glm.imat3x3, glm.mat3x3, glm.umat3x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
AnyAnyMat3x3 = Union[glm.dmat3x3, glm.imat3x3, glm.mat3x3, glm.umat3x3]
AnyAnyMatrix3x4 = Union[glm.dmat3x4, glm.imat3x4, glm.mat3x4, glm.umat3x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
AnyAnyMat3x4 = Union[glm.dmat3x4, glm.imat3x4, glm.mat3x4, glm.umat3x4]
AnyAnyMatrix3xAny = Union[glm.dmat3x2, glm.dmat3x3, glm.dmat3x4, glm.imat3x2, glm.imat3x3, glm.imat3x4, glm.mat3x2, glm.mat3x3, glm.mat3x4, glm.umat3x2, glm.umat3x3, glm.umat3x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
AnyAnyMat3xAny = Union[glm.dmat3x2, glm.dmat3x3, glm.dmat3x4, glm.imat3x2, glm.imat3x3, glm.imat3x4, glm.mat3x2, glm.mat3x3, glm.mat3x4, glm.umat3x2, glm.umat3x3, glm.umat3x4]
AnyAnyMatrix4x2 = Union[glm.dmat4x2, glm.imat4x2, glm.mat4x2, glm.umat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
AnyAnyMat4x2 = Union[glm.dmat4x2, glm.imat4x2, glm.mat4x2, glm.umat4x2]
AnyAnyMatrix4x3 = Union[glm.dmat4x3, glm.imat4x3, glm.mat4x3, glm.umat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
AnyAnyMat4x3 = Union[glm.dmat4x3, glm.imat4x3, glm.mat4x3, glm.umat4x3]
AnyAnyMatrix4x4 = Union[glm.dmat4x4, glm.imat4x4, glm.mat4x4, glm.umat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
AnyAnyMat4x4 = Union[glm.dmat4x4, glm.imat4x4, glm.mat4x4, glm.umat4x4]
AnyAnyMatrix4xAny = Union[glm.dmat4x2, glm.dmat4x3, glm.dmat4x4, glm.imat4x2, glm.imat4x3, glm.imat4x4, glm.mat4x2, glm.mat4x3, glm.mat4x4, glm.umat4x2, glm.umat4x3, glm.umat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
AnyAnyMat4xAny = Union[glm.dmat4x2, glm.dmat4x3, glm.dmat4x4, glm.imat4x2, glm.imat4x3, glm.imat4x4, glm.mat4x2, glm.mat4x3, glm.mat4x4, glm.umat4x2, glm.umat4x3, glm.umat4x4]
AnyAnyMatrixAnyx2 = Union[glm.dmat2x2, glm.dmat3x2, glm.dmat4x2, glm.imat2x2, glm.imat3x2, glm.imat4x2, glm.mat2x2, glm.mat3x2, glm.mat4x2, glm.umat2x2, glm.umat3x2, glm.umat4x2, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]]]
AnyAnyMatAnyx2 = Union[glm.dmat2x2, glm.dmat3x2, glm.dmat4x2, glm.imat2x2, glm.imat3x2, glm.imat4x2, glm.mat2x2, glm.mat3x2, glm.mat4x2, glm.umat2x2, glm.umat3x2, glm.umat4x2]
AnyAnyMatrixAnyx3 = Union[glm.dmat2x3, glm.dmat3x3, glm.dmat4x3, glm.imat2x3, glm.imat3x3, glm.imat4x3, glm.mat2x3, glm.mat3x3, glm.mat4x3, glm.umat2x3, glm.umat3x3, glm.umat4x3, Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]]]
AnyAnyMatAnyx3 = Union[glm.dmat2x3, glm.dmat3x3, glm.dmat4x3, glm.imat2x3, glm.imat3x3, glm.imat4x3, glm.mat2x3, glm.mat3x3, glm.mat4x3, glm.umat2x3, glm.umat3x3, glm.umat4x3]
AnyAnyMatrixAnyx4 = Union[glm.dmat2x4, glm.dmat3x4, glm.dmat4x4, glm.imat2x4, glm.imat3x4, glm.imat4x4, glm.mat2x4, glm.mat3x4, glm.mat4x4, glm.umat2x4, glm.umat3x4, glm.umat4x4, Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
AnyAnyMatAnyx4 = Union[glm.dmat2x4, glm.dmat3x4, glm.dmat4x4, glm.imat2x4, glm.imat3x4, glm.imat4x4, glm.mat2x4, glm.mat3x4, glm.mat4x4, glm.umat2x4, glm.umat3x4, glm.umat4x4]
AnyAnyMatrixAnyxAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4, glm.dmat3x2, glm.dmat3x3, glm.dmat3x4, glm.dmat4x2, glm.dmat4x3, glm.dmat4x4, glm.imat2x2, glm.imat2x3, glm.imat2x4, glm.imat3x2, glm.imat3x3, glm.imat3x4, glm.imat4x2, glm.imat4x3, glm.imat4x4, glm.mat2x2, glm.mat2x3, glm.mat2x4, glm.mat3x2, glm.mat3x3, glm.mat3x4, glm.mat4x2, glm.mat4x3, glm.mat4x4, glm.umat2x2, glm.umat2x3, glm.umat2x4, glm.umat3x2, glm.umat3x3, glm.umat3x4, glm.umat4x2, glm.umat4x3, glm.umat4x4, Tuple[Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]], Tuple[Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number], Tuple[Number, Number]], Tuple[Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number], Tuple[Number, Number, Number]], Tuple[Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number], Tuple[Number, Number, Number, Number]]]
AnyAnyMatAnyxAny = Union[glm.dmat2x2, glm.dmat2x3, glm.dmat2x4, glm.dmat3x2, glm.dmat3x3, glm.dmat3x4, glm.dmat4x2, glm.dmat4x3, glm.dmat4x4, glm.imat2x2, glm.imat2x3, glm.imat2x4, glm.imat3x2, glm.imat3x3, glm.imat3x4, glm.imat4x2, glm.imat4x3, glm.imat4x4, glm.mat2x2, glm.mat2x3, glm.mat2x4, glm.mat3x2, glm.mat3x3, glm.mat3x4, glm.mat4x2, glm.mat4x3, glm.mat4x4, glm.umat2x2, glm.umat2x3, glm.umat2x4, glm.umat3x2, glm.umat3x3, glm.umat3x4, glm.umat4x2, glm.umat4x3, glm.umat4x4]
AnyAnyMatrixSquare = Union[AnyAnyMatrix2x2, AnyAnyMatrix3x3, AnyAnyMatrix4x4]
D64Quaternion = Union[glm.dquat, Tuple[Number, Number, Number, Number]]
D64Quat = glm.dquat
DAnyQuaternion = Union[glm.dquat, Tuple[Number, Number, Number, Number]]
DAnyQuat = glm.dquat
F32Quaternion = Union[glm.quat, Tuple[Number, Number, Number, Number]]
F32Quat = glm.quat
FAnyQuaternion = Union[glm.quat, Tuple[Number, Number, Number, Number]]
FAnyQuat = glm.quat
AnyAnyQuaternion = Union[glm.dquat, glm.quat, Tuple[Number, Number, Number, Number]]
AnyAnyQuat = Union[glm.dquat, glm.quat]
FDAnyQuaternion = Union[FAnyQuaternion, DAnyQuaternion]
FDAnyQuaternionVector4 = Union[FDAnyVector4, FDAnyQuaternion]
__all__ = ['B8Vector1', 'B8Vec1', 'B8Vector2', 'B8Vec2', 'B8Vector3', 'B8Vec3', 'B8Vector4', 'B8Vec4', 'B8VectorAny', 'B8VecAny', 'BAnyVector1', 'BAnyVec1', 'BAnyVector2', 'BAnyVec2', 'BAnyVector3', 'BAnyVec3', 'BAnyVector4', 'BAnyVec4', 'BAnyVectorAny', 'BAnyVecAny', 'D64Vector1', 'D64Vec1', 'D64Vector2', 'D64Vec2', 'D64Vector3', 'D64Vec3', 'D64Vector4', 'D64Vec4', 'D64VectorAny', 'D64VecAny', 'DAnyVector1', 'DAnyVec1', 'DAnyVector2', 'DAnyVec2', 'DAnyVector3', 'DAnyVec3', 'DAnyVector4', 'DAnyVec4', 'DAnyVectorAny', 'DAnyVecAny', 'F32Vector1', 'F32Vec1', 'F32Vector2', 'F32Vec2', 'F32Vector3', 'F32Vec3', 'F32Vector4', 'F32Vec4', 'F32VectorAny', 'F32VecAny', 'FAnyVector1', 'FAnyVec1', 'FAnyVector2', 'FAnyVec2', 'FAnyVector3', 'FAnyVec3', 'FAnyVector4', 'FAnyVec4', 'FAnyVectorAny', 'FAnyVecAny', 'I8Vector1', 'I8Vec1', 'I8Vector2', 'I8Vec2', 'I8Vector3', 'I8Vec3', 'I8Vector4', 'I8Vec4', 'I8VectorAny', 'I8VecAny', 'I16Vector1', 'I16Vec1', 'I16Vector2', 'I16Vec2', 'I16Vector3', 'I16Vec3', 'I16Vector4', 'I16Vec4', 'I16VectorAny', 'I16VecAny', 'I32Vector1', 'I32Vec1', 'I32Vector2', 'I32Vec2', 'I32Vector3', 'I32Vec3', 'I32Vector4', 'I32Vec4', 'I32VectorAny', 'I32VecAny', 'I64Vector1', 'I64Vec1', 'I64Vector2', 'I64Vec2', 'I64Vector3', 'I64Vec3', 'I64Vector4', 'I64Vec4', 'I64VectorAny', 'I64VecAny', 'IAnyVector1', 'IAnyVec1', 'IAnyVector2', 'IAnyVec2', 'IAnyVector3', 'IAnyVec3', 'IAnyVector4', 'IAnyVec4', 'IAnyVectorAny', 'IAnyVecAny', 'U8Vector1', 'U8Vec1', 'U8Vector2', 'U8Vec2', 'U8Vector3', 'U8Vec3', 'U8Vector4', 'U8Vec4', 'U8VectorAny', 'U8VecAny', 'U16Vector1', 'U16Vec1', 'U16Vector2', 'U16Vec2', 'U16Vector3', 'U16Vec3', 'U16Vector4', 'U16Vec4', 'U16VectorAny', 'U16VecAny', 'U32Vector1', 'U32Vec1', 'U32Vector2', 'U32Vec2', 'U32Vector3', 'U32Vec3', 'U32Vector4', 'U32Vec4', 'U32VectorAny', 'U32VecAny', 'U64Vector1', 'U64Vec1', 'U64Vector2', 'U64Vec2', 'U64Vector3', 'U64Vec3', 'U64Vector4', 'U64Vec4', 'U64VectorAny', 'U64VecAny', 'UAnyVector1', 'UAnyVec1', 'UAnyVector2', 'UAnyVec2', 'UAnyVector3', 'UAnyVec3', 'UAnyVector4', 'UAnyVec4', 'UAnyVectorAny', 'UAnyVecAny', 'Any8Vector1', 'Any8Vec1', 'Any8Vector2', 'Any8Vec2', 'Any8Vector3', 'Any8Vec3', 'Any8Vector4', 'Any8Vec4', 'Any8VectorAny', 'Any8VecAny', 'Any16Vector1', 'Any16Vec1', 'Any16Vector2', 'Any16Vec2', 'Any16Vector3', 'Any16Vec3', 'Any16Vector4', 'Any16Vec4', 'Any16VectorAny', 'Any16VecAny', 'Any32Vector1', 'Any32Vec1', 'Any32Vector2', 'Any32Vec2', 'Any32Vector3', 'Any32Vec3', 'Any32Vector4', 'Any32Vec4', 'Any32VectorAny', 'Any32VecAny', 'Any64Vector1', 'Any64Vec1', 'Any64Vector2', 'Any64Vec2', 'Any64Vector3', 'Any64Vec3', 'Any64Vector4', 'Any64Vec4', 'Any64VectorAny', 'Any64VecAny', 'AnyAnyVector1', 'AnyAnyVec1', 'AnyAnyVector2', 'AnyAnyVec2', 'AnyAnyVector3', 'AnyAnyVec3', 'AnyAnyVector4', 'AnyAnyVec4', 'AnyAnyVectorAny', 'AnyAnyVecAny', 'FDAnyVectorAny', 'FDAnyVector1', 'FDAnyVector2', 'FDAnyVector3', 'FDAnyVector4', 'IUAnyVector1', 'IUAnyVector2', 'IUAnyVector3', 'IUAnyVector4', 'D64Matrix2x2', 'D64Mat2x2', 'D64Matrix2x3', 'D64Mat2x3', 'D64Matrix2x4', 'D64Mat2x4', 'D64Matrix2xAny', 'D64Mat2xAny', 'D64Matrix3x2', 'D64Mat3x2', 'D64Matrix3x3', 'D64Mat3x3', 'D64Matrix3x4', 'D64Mat3x4', 'D64Matrix3xAny', 'D64Mat3xAny', 'D64Matrix4x2', 'D64Mat4x2', 'D64Matrix4x3', 'D64Mat4x3', 'D64Matrix4x4', 'D64Mat4x4', 'D64Matrix4xAny', 'D64Mat4xAny', 'D64MatrixAnyx2', 'D64MatAnyx2', 'D64MatrixAnyx3', 'D64MatAnyx3', 'D64MatrixAnyx4', 'D64MatAnyx4', 'D64MatrixAnyxAny', 'D64MatAnyxAny', 'DAnyMatrix2x2', 'DAnyMat2x2', 'DAnyMatrix2x3', 'DAnyMat2x3', 'DAnyMatrix2x4', 'DAnyMat2x4', 'DAnyMatrix2xAny', 'DAnyMat2xAny', 'DAnyMatrix3x2', 'DAnyMat3x2', 'DAnyMatrix3x3', 'DAnyMat3x3', 'DAnyMatrix3x4', 'DAnyMat3x4', 'DAnyMatrix3xAny', 'DAnyMat3xAny', 'DAnyMatrix4x2', 'DAnyMat4x2', 'DAnyMatrix4x3', 'DAnyMat4x3', 'DAnyMatrix4x4', 'DAnyMat4x4', 'DAnyMatrix4xAny', 'DAnyMat4xAny', 'DAnyMatrixAnyx2', 'DAnyMatAnyx2', 'DAnyMatrixAnyx3', 'DAnyMatAnyx3', 'DAnyMatrixAnyx4', 'DAnyMatAnyx4', 'DAnyMatrixAnyxAny', 'DAnyMatAnyxAny', 'F32Matrix2x2', 'F32Mat2x2', 'F32Matrix2x3', 'F32Mat2x3', 'F32Matrix2x4', 'F32Mat2x4', 'F32Matrix2xAny', 'F32Mat2xAny', 'F32Matrix3x2', 'F32Mat3x2', 'F32Matrix3x3', 'F32Mat3x3', 'F32Matrix3x4', 'F32Mat3x4', 'F32Matrix3xAny', 'F32Mat3xAny', 'F32Matrix4x2', 'F32Mat4x2', 'F32Matrix4x3', 'F32Mat4x3', 'F32Matrix4x4', 'F32Mat4x4', 'F32Matrix4xAny', 'F32Mat4xAny', 'F32MatrixAnyx2', 'F32MatAnyx2', 'F32MatrixAnyx3', 'F32MatAnyx3', 'F32MatrixAnyx4', 'F32MatAnyx4', 'F32MatrixAnyxAny', 'F32MatAnyxAny', 'FAnyMatrix2x2', 'FAnyMat2x2', 'FAnyMatrix2x3', 'FAnyMat2x3', 'FAnyMatrix2x4', 'FAnyMat2x4', 'FAnyMatrix2xAny', 'FAnyMat2xAny', 'FAnyMatrix3x2', 'FAnyMat3x2', 'FAnyMatrix3x3', 'FAnyMat3x3', 'FAnyMatrix3x4', 'FAnyMat3x4', 'FAnyMatrix3xAny', 'FAnyMat3xAny', 'FAnyMatrix4x2', 'FAnyMat4x2', 'FAnyMatrix4x3', 'FAnyMat4x3', 'FAnyMatrix4x4', 'FAnyMat4x4', 'FAnyMatrix4xAny', 'FAnyMat4xAny', 'FAnyMatrixAnyx2', 'FAnyMatAnyx2', 'FAnyMatrixAnyx3', 'FAnyMatAnyx3', 'FAnyMatrixAnyx4', 'FAnyMatAnyx4', 'FAnyMatrixAnyxAny', 'FAnyMatAnyxAny', 'I32Matrix2x2', 'I32Mat2x2', 'I32Matrix2x3', 'I32Mat2x3', 'I32Matrix2x4', 'I32Mat2x4', 'I32Matrix2xAny', 'I32Mat2xAny', 'I32Matrix3x2', 'I32Mat3x2', 'I32Matrix3x3', 'I32Mat3x3', 'I32Matrix3x4', 'I32Mat3x4', 'I32Matrix3xAny', 'I32Mat3xAny', 'I32Matrix4x2', 'I32Mat4x2', 'I32Matrix4x3', 'I32Mat4x3', 'I32Matrix4x4', 'I32Mat4x4', 'I32Matrix4xAny', 'I32Mat4xAny', 'I32MatrixAnyx2', 'I32MatAnyx2', 'I32MatrixAnyx3', 'I32MatAnyx3', 'I32MatrixAnyx4', 'I32MatAnyx4', 'I32MatrixAnyxAny', 'I32MatAnyxAny', 'IAnyMatrix2x2', 'IAnyMat2x2', 'IAnyMatrix2x3', 'IAnyMat2x3', 'IAnyMatrix2x4', 'IAnyMat2x4', 'IAnyMatrix2xAny', 'IAnyMat2xAny', 'IAnyMatrix3x2', 'IAnyMat3x2', 'IAnyMatrix3x3', 'IAnyMat3x3', 'IAnyMatrix3x4', 'IAnyMat3x4', 'IAnyMatrix3xAny', 'IAnyMat3xAny', 'IAnyMatrix4x2', 'IAnyMat4x2', 'IAnyMatrix4x3', 'IAnyMat4x3', 'IAnyMatrix4x4', 'IAnyMat4x4', 'IAnyMatrix4xAny', 'IAnyMat4xAny', 'IAnyMatrixAnyx2', 'IAnyMatAnyx2', 'IAnyMatrixAnyx3', 'IAnyMatAnyx3', 'IAnyMatrixAnyx4', 'IAnyMatAnyx4', 'IAnyMatrixAnyxAny', 'IAnyMatAnyxAny', 'U32Matrix2x2', 'U32Mat2x2', 'U32Matrix2x3', 'U32Mat2x3', 'U32Matrix2x4', 'U32Mat2x4', 'U32Matrix2xAny', 'U32Mat2xAny', 'U32Matrix3x2', 'U32Mat3x2', 'U32Matrix3x3', 'U32Mat3x3', 'U32Matrix3x4', 'U32Mat3x4', 'U32Matrix3xAny', 'U32Mat3xAny', 'U32Matrix4x2', 'U32Mat4x2', 'U32Matrix4x3', 'U32Mat4x3', 'U32Matrix4x4', 'U32Mat4x4', 'U32Matrix4xAny', 'U32Mat4xAny', 'U32MatrixAnyx2', 'U32MatAnyx2', 'U32MatrixAnyx3', 'U32MatAnyx3', 'U32MatrixAnyx4', 'U32MatAnyx4', 'U32MatrixAnyxAny', 'U32MatAnyxAny', 'UAnyMatrix2x2', 'UAnyMat2x2', 'UAnyMatrix2x3', 'UAnyMat2x3', 'UAnyMatrix2x4', 'UAnyMat2x4', 'UAnyMatrix2xAny', 'UAnyMat2xAny', 'UAnyMatrix3x2', 'UAnyMat3x2', 'UAnyMatrix3x3', 'UAnyMat3x3', 'UAnyMatrix3x4', 'UAnyMat3x4', 'UAnyMatrix3xAny', 'UAnyMat3xAny', 'UAnyMatrix4x2', 'UAnyMat4x2', 'UAnyMatrix4x3', 'UAnyMat4x3', 'UAnyMatrix4x4', 'UAnyMat4x4', 'UAnyMatrix4xAny', 'UAnyMat4xAny', 'UAnyMatrixAnyx2', 'UAnyMatAnyx2', 'UAnyMatrixAnyx3', 'UAnyMatAnyx3', 'UAnyMatrixAnyx4', 'UAnyMatAnyx4', 'UAnyMatrixAnyxAny', 'UAnyMatAnyxAny', 'AnyAnyMatrix2x2', 'AnyAnyMat2x2', 'AnyAnyMatrix2x3', 'AnyAnyMat2x3', 'AnyAnyMatrix2x4', 'AnyAnyMat2x4', 'AnyAnyMatrix2xAny', 'AnyAnyMat2xAny', 'AnyAnyMatrix3x2', 'AnyAnyMat3x2', 'AnyAnyMatrix3x3', 'AnyAnyMat3x3', 'AnyAnyMatrix3x4', 'AnyAnyMat3x4', 'AnyAnyMatrix3xAny', 'AnyAnyMat3xAny', 'AnyAnyMatrix4x2', 'AnyAnyMat4x2', 'AnyAnyMatrix4x3', 'AnyAnyMat4x3', 'AnyAnyMatrix4x4', 'AnyAnyMat4x4', 'AnyAnyMatrix4xAny', 'AnyAnyMat4xAny', 'AnyAnyMatrixAnyx2', 'AnyAnyMatAnyx2', 'AnyAnyMatrixAnyx3', 'AnyAnyMatAnyx3', 'AnyAnyMatrixAnyx4', 'AnyAnyMatAnyx4', 'AnyAnyMatrixAnyxAny', 'AnyAnyMatAnyxAny', 'AnyAnyMatrixSquare', 'D64Quaternion', 'D64Quat', 'DAnyQuaternion', 'DAnyQuat', 'F32Quaternion', 'F32Quat', 'FAnyQuaternion', 'FAnyQuat', 'AnyAnyQuaternion', 'AnyAnyQuat', 'FDAnyQuaternion', 'FDAnyQuaternionVector4']
