#define PyGLM_LICENSE "PyGLM license information:\n"\
"\n"\
"================================================================================\n"\
"OpenGL Mathematics (GLM)\n"\
"--------------------------------------------------------------------------------\n"\
"This is a redistribution of OpenGL Mathematics (GLM), which is licensed under\n"\
"\n"\
"================================================================================\n"\
"The MIT License\n"\
"--------------------------------------------------------------------------------\n"\
"Copyright (c) 2005 G-Truc Creation\n"\
"\n"\
"Permission is hereby granted, PyMem_Free of charge, to any person obtaining a copy\n"\
"of this software and associated documentation files (the \"Software\"), to deal\n"\
"in the Software without restriction, including without limitation the rights\n"\
"to use, copy, modify, merge, publish, distribute, sublicense, and/or sell\n"\
"copies of the Software, and to permit persons to whom the Software is\n"\
"furnished to do so, subject to the following conditions:\n"\
"\n"\
"The above copyright notice and this permission notice shall be included in\n"\
"all copies or substantial portions of the Software.\n"\
"\n"\
"THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n"\
"IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n"\
"FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n"\
"AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n"\
"LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\n"\
"OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\n"\
"THE SOFTWARE.\n"\
"\n"\
"================================================================================\n"\
"PyGLM\n"\
"--------------------------------------------------------------------------------\n"\
"PyGLM is licensed under the zlib/libpng License\n"\
"\n"\
"================================================================================\n"\
"zlib/libpng license\n"\
"--------------------------------------------------------------------------------\n"\
"Copyright (c) 2017 Zuzu_Typ\n"\
"\n"\
"This software is provided 'as-is', without any express or implied\n"\
"warranty. In no event will the authors be held liable for any damages\n"\
"arising from the use of this software.\n"\
"\n"\
"Permission is granted to anyone to use this software for any purpose,\n"\
"including commercial applications, and to alter it and redistribute it\n"\
"freely, subject to the following restrictions:\n"\
"\n"\
"1. The origin of this software must not be misrepresented; you must not\n"\
"   claim that you wrote the original software. If you use this software\n"\
"   in a product, an acknowledgment in the product documentation would be\n"\
"   appreciated but is not required.\n"\
"2. Altered source versions must be plainly marked as such, and must not be\n"\
"   misrepresented as being the original software.\n"\
"3. This notice may not be removed or altered from any source distribution."
