#ifndef TYPES_MAT_DOUBLE_ALL_H
#define TYPES_MAT_DOUBLE_ALL_H

#include "mat2x2.h"
#include "mat2x3.h"
#include "mat2x4.h"

#include "mat3x2.h"
#include "mat3x3.h"
#include "mat3x4.h"

#include "mat4x2.h"
#include "mat4x3.h"
#include "mat4x4.h"

#endif